/* gdkDPS.h
 *
 * Copyright (C) 1997, 1998, 1999 GYVE Development Team
 *
 * Author:  Terumoto 'tel' HAYAKAWA <hayakawa@cv.cs.ritsumei.ac.jp>
 * Maintainer: Masatake YAMATO <masata-y@is.aist-nara.ac.jp>
 * Created: 
 *
 * This file is part of gtkDPS Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef GDK__DPS_H
#define GDK__DPS_H

#include <gdk/gdk.h>
#include <gdk/gdktypes.h>
#include <DPS/dpsclient.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* Initialize */
void gdk_dps_init (int *argc, char ***argv);

/* Declare types */
typedef struct _GdkDPSCoordtr   GdkDPSCoordtr;
typedef struct _GdkDPSPoint     GdkDPSPoint;
typedef struct _GdkDPSRectangle GdkDPSRectangle;
typedef struct _GdkDPSContext   GdkDPSContext;

/* Product */
typedef enum 
{
  GDK_DPS_AGENT_PRODUCT_ADOBE_DPS = 1,
  GDK_DPS_AGENT_PRODUCT_GNU_DGS,
  GDK_DPS_AGENT_PRODUCT_ALADDIN_DGS
} GdkDPSAgentProduct;
GdkDPSAgentProduct gdk_dps_get_agent_product();

/* Agent revision */
typedef enum 
{
  GDK_DPS_AGENT_REVISION_5 = 1,
  GDK_DPS_AGENT_REVISION_6

} GdkDPSAgentRevision;
GdkDPSAgentRevision gdk_dps_get_agent_revision();

/* World */
typedef enum
{
  GDK_DPS_WORLD_DPS,
  GDK_DPS_WORLD_X
} GdkDPSWorld; 

typedef enum
{
  GDK_DPS_TRDIR_DPS2X,
  GDK_DPS_TRDIR_X2DPS
} GdkDPSTrDirection; 

/* Coord translator */
struct _GdkDPSCoordtr
{
  gfloat ctm[6];
  gfloat invctm[6];
  gint x_offset;
  gint y_offset;
  gboolean data_dirty;
};

void gdk_dps_coordtr_init(GdkDPSCoordtr*);
void gdk_dps_coordtr_make_dirty(GdkDPSCoordtr* );
gboolean gdk_dps_coordtr_is_dirty(GdkDPSCoordtr*);

/* Point */
struct _GdkDPSPoint
{
  gfloat x;
  gfloat y;
};
void gdk_dps_coordtr_point(GdkDPSCoordtr* coordtr,
			   GdkDPSTrDirection trdir, 
			   gpointer src, 
			   gpointer dist);

void gdk_dps_coordtr_point_x2dps(GdkDPSCoordtr* coordtr,
				 GdkPoint * x_src,
				 GdkDPSPoint* dps_dist);
void gdk_dps_coordtr_point_dps2x(GdkDPSCoordtr* coordtr,
				 GdkDPSPoint * dps_src,
				 GdkPoint* x_dist);

/* Rectangle */
struct _GdkDPSRectangle
{
  gfloat x;
  gfloat y;
  gfloat width;
  gfloat height;
};
void gdk_dps_coordtr_rectangle(GdkDPSCoordtr* coordtr,
			       GdkDPSTrDirection trdir, 
			       gpointer src, 
			       gpointer dist);

void gdk_dps_coordtr_rectangle_x2dps(GdkDPSCoordtr*,
      			       GdkRectangle * x_src,
      			       GdkDPSRectangle* dps_dist);

void gdk_dps_coordtr_rectangle_dps2x(GdkDPSCoordtr*,
				     GdkDPSRectangle * dps_src,
				     GdkRectangle * x_dist);

gfloat gdk_dps_coordtr_width(GdkDPSCoordtr* coordtr,
			     GdkDPSTrDirection trdir, 
			     gfloat width);
gfloat gdk_dps_coordtr_height(GdkDPSCoordtr* coordtr,
			      GdkDPSTrDirection trdir, 
			      gfloat height);


/* Context */
struct _GdkDPSContext
{
  DPSContext raw_ctxt;
  GdkGC* gc;
};

/* Create context */
GdkDPSContext* gdk_dps_context_get_shared (void);
GdkDPSContext* gdk_dps_context_new (GdkWindow *);

/* Query */
gboolean gdk_dps_context_is_shared (GdkDPSContext*);

/* Destroy context */
void           gdk_dps_context_destroy    (GdkDPSContext *);

/* Access struct internal variables */
GdkGC*     gdk_dps_context_get_gc (GdkDPSContext *);
DPSContext gdk_dps_context_get_raw_context(GdkDPSContext *);

/* Block */
#define gdk_dps_context_begin(ctxt)                           \
{                                                             \
  DPSContext raw_ctxt = gdk_dps_context_get_raw_context(ctxt);
#define  gdk_dps_context_end(ctxt) \
}

/* Coord */
void gdk_dps_context_update_coordtr(GdkDPSContext * ctxt, 
				    GdkDPSCoordtr * coordtr);
gboolean gdk_dps_context_try_update_coordtr(GdkDPSContext * ctxt, 
					    GdkDPSCoordtr * coordtr);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* GDK__DPS_CONTEXT_H */
